function sac_write_n

% Writes the sorted data of a certain channel into a *.nev or *.mat file.

global FileInfo;
global ClusterInfo;
global WaveformInfo;
global Handles;

tic
if exist('WaveformInfo') & isfield(WaveformInfo, 'Waveforms')
    store=WaveformInfo.Waveforms;%save Waveforms in temporary variable
else
    store=[];
end

[g,d]=size(ClusterInfo.Centers);% g - centers count
nu=ClusterInfo.nu;
Pj=sparse(ClusterInfo.Proportions);
detSigma=[];
CinvSqrt=[];
for i=1:g
    CinvSqrt{i}=sqrtm(pinv(ClusterInfo.Sigma{i}));
    detSigma(i)=1/sqrt(det(ClusterInfo.Sigma{i}));
end
detSigma=diag(sparse(detSigma));
if ~isreal(detSigma)
    warning('Singular covariance')
end
%-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
filenum=length(FileInfo);
% ChannelInfo= get(Handles.channel,'UserData');% gives the current channel number
% ChannelNumber=ChannelInfo{1}(get(Handles.channel,'Value'));
ChannelNumber=ClusterInfo.ChannelNumber;
SamplingRate=1000*FileInfo(1).SamplingRate; % sample rate given to write_nev is in Hz
switch(FileInfo(1).load)
    case 'Neuro'
        noisecluster=255;
        for j=1:filenum %Loop across files
            isnew='no';
            if isempty(FileInfo(j).write),
                [pathstr,name,ext,versn]=fileparts(FileInfo.filename);
                [filename, pathname] = uiputfile({'*.nev';'*.mat'},['Write "',name,ext,'" to...']) ; % recieves file name
                if filename==0,
                    return;
                end
                clear pathstr name ext versn
                [a,type]=strtok(filename,'.')  ;
                type=type(2:end);
                FileInfo(j).write.type=type;  % filetype to write to nev or mat
                FileInfo(j).write.filename=filename;
                FileInfo(j).write.pathname=pathname;
                isnew='yes';
            end
            %             [nsresult] = ns_SetLibrary(FileInfo(1).dllname) ;              % opening dll
            Waves=[];  % cleared for every new file
            %             [nsresult, hfile] = ns_OpenFile( FileInfo(j).filename);% opening the file
            FLimits=length(FileInfo(1).ChannelLimits)>=ChannelNumber; % checks that limits vector is at least as long as channel num.
            if  FLimits
                Limits=1-isempty(FileInfo(1).ChannelLimits(ChannelNumber).push); % if no limits were set, push field is empty
                if Limits
                    X=FileInfo(1).ChannelLimits(ChannelNumber).x;
                    Max=FileInfo(1).ChannelLimits(ChannelNumber).Max;
                    Min=FileInfo(1).ChannelLimits(ChannelNumber).Min;
                end
            end
            EntityID=FileInfo(j).ChannelEntityID(ChannelNumber);    % entityID of channel c
            num_to_read=FileInfo(j).SpikesNumber(ChannelNumber);     % num. of spikes on channel c
            Units=[];
            n=min(3000,num_to_read); %# of waveforms read at a time
            howmany=ceil(num_to_read/n);%# of read cycles
            current=1;
            h=waitbar(0,['Writing Channel ',num2str(ChannelNumber),' to File']);
            for i=1:howmany
                WaveformInfo.Waveforms=[];
                % Read in the Waveforms
                if FLimits==1&Limits==1,
                    n=min(3000,num_to_read);
                end
                if i==howmany   % last group is smaller
                    n=num_to_read-current+1; %line changed August 12, 2003
                end
                waitbar(i/howmany,h);
                [ns_RESULT, TimeStamp, Data, SampleCount, UnitID] = ns_GetSegmentData(FileInfo(j).hfile, EntityID,[current:n+current-1]);
                Data=Data';
                WaveformInfo.Waveforms=Data;
                sac_align(size(ClusterInfo.Centers,2));
                current=current+n;
                % Limits Cleaning:
                if FLimits==1&Limits==1,
                    MinCheck=prod(double(WaveformInfo.Waveforms(:,X)>=repmat(Min,length(WaveformInfo.Waveforms(:,1)),1)),2); % ones where MinCheck ok
                    MaxCheck=prod(double(WaveformInfo.Waveforms(:,X)<=repmat(Max,length(WaveformInfo.Waveforms(:,1)),1)),2); % ones where MaxCheck ok
                    good=find(MinCheck.*MaxCheck);
                    n=length(good);% waveform num may be reduced
                else
                    good=[1:n];% if no limits were set, all spikes are saved
                end
                % %%% final n for current loop is set
                %   Calculate the classification
                M=zeros(n,g);
                for k=1:g
                    diffs=WaveformInfo.Waveforms(good,:)-ones(n,1)*ClusterInfo.Centers(k,:);
                    M(:,k)=sum(((CinvSqrt{k}*diffs').^2))'; %Mahalanobis distances
                end
                Prob=exp(-(nu+d)*log(1+M/nu)/2)*detSigma;
                Z=Prob*diag(Pj)./(sum(Prob*diag(Pj),2)*ones(1,g));
                U=(nu+d)./(nu+M);
                [Y,Classification]=max((Z)',[],1);
                Classification=ClusterInfo.Units(Classification);
                Classification(find(max((Z.*U)',[],1)<0.5))=noisecluster; %outliers are assigned to noise cluster
                % saving part   :
                %-------------------------
                switch(FileInfo(j).write.type)
                    case 'nev'
                        if max(max(abs(Data(good',:))))<10,
                            sac_write_nev(1000*Data(good',:),1000*TimeStamp(good),SamplingRate,...
                                FileInfo(1).nVperBit(ClusterInfo.ChannelNumber)/1000,Classification,ChannelNumber,j,isnew);
                        else
                            sac_write_nev(Data(good',:),1000*TimeStamp(good),SamplingRate,...
                                FileInfo(1).nVperBit(ClusterInfo.ChannelNumber),Classification,ChannelNumber,j,isnew);
                        end
                        isnew='no';
                    case 'mat'
                        switch(isnew)
                            case 'yes'
                                waves.Channel(1:n)=uint8(ChannelNumber); % allow up to 255 channels
                                waves.nVperBit=FileInfo(1).nVperBit(ClusterInfo.ChannelNumber);
                                waves.SR=SamplingRate/1000; %   SR in KHz
                                waves.UnitID(1:n)=uint8(Classification');
                                waves.spikes=Data(good',:);
                                waves.ts=TimeStamp(good); % Neuroshare reads times in seconds
                                save([FileInfo(j).write.pathname FileInfo(j).write.filename],'waves');
                                isnew='no';
                            case 'no'
                                load ([FileInfo(j).write.pathname FileInfo(j).write.filename]);
                                waves.Channel(end+1:end+n)=uint8(ChannelNumber); % allow up to 255 channels
                                waves.UnitID(end+1:end+n)=uint8(Classification');
                                waves.spikes(end+1:end+n,:)=Data(good',:);
                                waves.ts(end+1:end+n)=TimeStamp(good);
                                save([FileInfo(j).write.pathname FileInfo(j).write.filename],'waves');
                        end
                end
            end % howmany end
            close(h)
        end
        WaveformInfo.Waveforms=store;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    case 'Engine'
        noisecluster=255;
        for j=1:filenum %Loop across files
            Waves=[];  % cleared for every new file
            isnew='no';
            if isempty(FileInfo(j).write),
                [pathstr,name,ext,versn]=fileparts(FileInfo.filename);
                [filename, pathname] = uiputfile({'*.nev';'*.mat'},['Write "',name,ext,'" to...']) ; % recieves file name
                if filename==0,
                    return;
                end
                clear pathstr name ext versn
                [a,type]=strtok(filename,'.')  ;
                type=type(2:end);
                FileInfo(j).write.type=type;  % filetype to write to nev or mat
                FileInfo(j).write.filename=filename;
                FileInfo(j).write.pathname=pathname;
                isnew='yes';
            end
            FLimits=length(FileInfo(1).ChannelLimits)>=ChannelNumber;
            if  FLimits
                Limits=1-isempty(FileInfo(1).ChannelLimits(ChannelNumber).push); % if no limits were set, push field is empty
                if Limits
                    X=FileInfo(1).ChannelLimits(ChannelNumber).x;
                    Max=FileInfo(1).ChannelLimits(ChannelNumber).Max;
                    Min=FileInfo(1).ChannelLimits(ChannelNumber).Min;
                end
            end
            num_to_read=FileInfo(j).SpikesNumber(ChannelNumber);     % num. of spikes on channel c
            Units=[];
            n=min(3000,num_to_read); %# of waveforms read at a time
            howmany=ceil(num_to_read/n);%# of read cycles
            current=1;
            h=waitbar(0,['Writing Channel ',num2str(ChannelNumber),' to File']);
            for i=1:howmany
                WaveformInfo.Waveforms=[];
                % Read in the Waveforms
                if FLimits==1&Limits==1,
                    n=min(3000,num_to_read);
                end
                if i==howmany% last group is smaller
                    n=num_to_read-current+1; %line changed August 12, 2003
                end
                waitbar(i/howmany,h);
                TimeStamp=sac_engine(FileInfo(1).Fhandle,[j,double(ChannelNumber),current:n+current-1],'times');% time in seconds
                Data=sac_engine(FileInfo(1).Fhandle,[j,double(ChannelNumber),current:n+current-1],'waveforms'); % evry row is a spike
                WaveformInfo.Waveforms=Data;
                sac_align(size(ClusterInfo.Centers,2));
                current=current+n;                
                % Limits Cleaning:
                if FLimits==1&Limits==1,
                    MinCheck=prod(double(WaveformInfo.Waveforms(:,X)>=repmat(Min,length(WaveformInfo.Waveforms(:,1)),1)),2); % ones where MinCheck ok
                    MaxCheck=prod(double(WaveformInfo.Waveforms(:,X)<=repmat(Max,length(WaveformInfo.Waveforms(:,1)),1)),2); % ones where MaxCheck ok
                    good=find(MinCheck.*MaxCheck);
                    n=length(good);% waveform num may be reduced
                else
                    good=[1:n];% if no limits were set, all spikes are saved
                end
                % %%% final n for current loop is set
                %   Calculate the classification
                M=zeros(n,g);
                for k=1:g
                    diffs=WaveformInfo.Waveforms(good,:)-ones(n,1)*ClusterInfo.Centers(k,:);
                    M(:,k)=sum(((CinvSqrt{k}*diffs').^2))'; %Mahalanobis distances
                end
                Prob=exp(-(nu+d)*log(1+M/nu)/2)*detSigma;
                Z=Prob*diag(Pj)./(sum(Prob*diag(Pj),2)*ones(1,g));
                U=(nu+d)./(nu+M);
                [Y,Classification]=max((Z)',[],1);
                Classification=ClusterInfo.Units(Classification);
                Classification(find(max((Z.*U)',[],1)<0.5))=noisecluster; %outliers are assigned to noise cluster
                % saving part   :
                %-------------------------
                switch(FileInfo(j).write.type)
                    case 'nev'
                        if max(max(abs(Data(good',:))))<10,
                            sac_write_nev(1000*Data(good',:),1000*TimeStamp(good),SamplingRate,...
                                FileInfo(1).nVperBit(ClusterInfo.ChannelNumber)/1000,Classification,ChannelNumber,j,isnew);
                        else
                            sac_write_nev(Data(good',:),1000*TimeStamp(good),SamplingRate,...
                                FileInfo(1).nVperBit(ClusterInfo.ChannelNumber),Classification,ChannelNumber,j,isnew);
                        end
                        isnew='no';
                    case 'mat'
                        switch(isnew)
                            case 'yes'
                                waves.Channel(1:n)=uint8(ChannelNumber); % allow up to 255 channels
                                waves.nVperBit=FileInfo(1).nVperBit(ClusterInfo.ChannelNumber);
                                waves.SR=SamplingRate/1000; %   SR in KHz
                                waves.UnitID(1:n)=uint8(Classification');
                                waves.spikes=Data(good',:);
                                waves.ts=TimeStamp(good); % Neuroshare reads times in seconds
                                save([FileInfo(j).write.pathname FileInfo(j).write.filename],'waves');
                                isnew='no';
                            case 'no'
                                load ([FileInfo(j).write.pathname FileInfo(j).write.filename]);
                                waves.Channel(end+1:end+n)=uint8(ChannelNumber); % allow up to 255 channels
                                waves.UnitID(end+1:end+n)=uint8(Classification');
                                waves.spikes(end+1:end+n,:)=Data(good',:);
                                waves.ts(end+1:end+n)=TimeStamp(good);
                                save([FileInfo(j).write.pathname FileInfo(j).write.filename],'waves');
                        end % inner switch end
                end  % outer switch end
            end % howmany end
            close(h)
        end
        WaveformInfo.Waveforms=store;
end

% adding indication that the channel has been written:
ChannelString=get(Handles.channel,'String');% get the string
ChannelString{ChannelNumber}=['[w] ',ChannelString{ChannelNumber}];
set(Handles.channel,'String',ChannelString);
WriteToc=toc